/* Copyright (C) 1997,1998,1999,2000,2001,2006 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, write to the Free
   Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
   02110-1301 USA.  */

/*
 *	ISO C99: 7.18 Integer types <stdint.h>
 */

/*
 * ISSUE:
 * This file is based on the system stdint.h for avr
 */
#ifndef _STDINT_AVR_H
#define _STDINT_AVR_H	1

#include "custom_limits.h"

/* Exact integral types.  */

/* Signed.  */

typedef signed char				int8_t;
typedef int					int16_t;
typedef long int				int32_t;
typedef long long int				int64_t;

/* Unsigned.  */
typedef unsigned char				uint8_t;
typedef unsigned int				uint16_t;
typedef unsigned long int			uint32_t;
typedef unsigned long long int			uint64_t;

#endif /* stdint_avr.h */
